"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var mockery_1 = __importDefault(require("mockery"));
var _AnimatedValue_1 = __importDefault(require("./_AnimatedValue"));
mockery_1.default.enable({
    warnOnUnregistered: false
});
var getValue = function (node) {
    if (typeof node === 'number') {
        return node;
    }
    return node.__value;
};
mockery_1.default.registerMock('react-native-reanimated', {
    Value: _AnimatedValue_1.default,
    Node: _AnimatedValue_1.default,
    add: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) + getValue(b));
    },
    sub: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) - getValue(b));
    },
    divide: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) / getValue(b));
    },
    multiply: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) * getValue(b));
    },
    sin: function (a) { return new _AnimatedValue_1.default(Math.sin(getValue(a))); },
    tan: function (a) { return new _AnimatedValue_1.default(Math.tan(getValue(a))); },
    cos: function (a) { return new _AnimatedValue_1.default(Math.cos(getValue(a))); },
    sqrt: function (a) { return new _AnimatedValue_1.default(Math.sqrt(getValue(a))); },
    min: function () {
        var a = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            a[_i] = arguments[_i];
        }
        return new _AnimatedValue_1.default(Math.min.apply(Math, a.map(function (b) { return getValue(b); })));
    },
    max: function () {
        var a = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            a[_i] = arguments[_i];
        }
        return new _AnimatedValue_1.default(Math.max.apply(Math, a.map(function (b) { return getValue(b); })));
    },
    pow: function () {
        var a = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            a[_i] = arguments[_i];
        }
        return new _AnimatedValue_1.default(a.reduce(function (b, c, index) { return (index > 0 ? Math.pow(getValue(b), getValue(c)) : c); }, 0));
    },
    and: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) && getValue(b));
    },
    set: function (a, b) { return getValue(b); },
    or: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) || getValue(b));
    },
    modulo: function (a, b) {
        return new _AnimatedValue_1.default(getValue(a) % getValue(b));
    },
    exp: function (a) { return new _AnimatedValue_1.default(Math.exp(getValue(a))); },
    asin: function (a) { return new _AnimatedValue_1.default(Math.asin(getValue(a))); },
    atan: function (a) { return new _AnimatedValue_1.default(Math.atan(getValue(a))); },
    acos: function (a) { return new _AnimatedValue_1.default(Math.acos(getValue(a))); },
    floor: function (a) { return new _AnimatedValue_1.default(Math.floor(getValue(a))); },
    abs: function (a) { return new _AnimatedValue_1.default(Math.abs(getValue(a))); },
    round: function (a) { return new _AnimatedValue_1.default(Math.round(getValue(a))); },
    ceil: function (a) { return new _AnimatedValue_1.default(Math.ceil(getValue(a))); },
    acc: function (a) { return 'acc_mock' + getValue(a); },
    diff: function (a) { return 'diff_mock' + getValue(a); },
    concat: function (a, b) { return getValue(a) + b; },
    defined: function (a) { return 'defined_mock' + getValue(a); },
    eq: function (a, b) {
        return new _AnimatedValue_1.default(Number(getValue(a) === getValue(b)));
    },
    neq: function (a, b) {
        return new _AnimatedValue_1.default(Number(getValue(a) !== getValue(b)));
    },
    lessThan: function (a, b) {
        return new _AnimatedValue_1.default(Number(getValue(a) < getValue(b)));
    },
    greaterThan: function (a, b) {
        return new _AnimatedValue_1.default(Number(getValue(a) > getValue(b)));
    },
    greaterOrEq: function (a, b) {
        return new _AnimatedValue_1.default(Number(getValue(a) >= getValue(b)));
    },
    lessOrEq: function (a, b) {
        return new _AnimatedValue_1.default(Number(getValue(a) <= getValue(b)));
    },
    not: function (a) { return new _AnimatedValue_1.default(Number(!getValue(a))); },
    cond: function (a, b, c) {
        return new _AnimatedValue_1.default(getValue(a) ? getValue(b) : getValue(c));
    }
});
