"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var reduce_ast_1 = __importDefault(require("./reduce-ast"));
var simple_math_ast_1 = __importDefault(require("./simple-math-ast"));
var react_native_reanimated_1 = __importDefault(require("react-native-reanimated"));
var InvalidExpressionError_1 = require("./InvalidExpressionError");
var log_prefix_1 = __importDefault(require("./log-prefix"));
var makeAst = function (args) {
    var placeholder = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        placeholder[_i - 1] = arguments[_i];
    }
    var variableMap = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
    if (args.length > variableMap.length) {
        throw new TypeError("Not more than " + variableMap.length + " variables are supported per expression.");
    }
    var intersect = [];
    var variables = {};
    for (var i = 0; i < args.length; i++) {
        intersect.push(args[i]);
        if (i !== args.length - 1) {
            var variable = variableMap[i];
            intersect.push(variable);
            variables[variable] = placeholder[i];
        }
        intersect = intersect.filter(Boolean);
    }
    var ast = simple_math_ast_1.default(intersect.join(''), variables);
    return [ast, variables];
};
var validateArgs = function (placeholders) {
    var check = [
        function (p) { return p instanceof react_native_reanimated_1.default.Node; },
        function (p) { return typeof p === 'number'; },
        function (p) { return p instanceof react_native_reanimated_1.default.Value; }
    ];
    var _loop_1 = function (placeholder) {
        if (!check.concat([
            function (p) {
                return Array.isArray(p) && p.every(function (_p) { return check.some(function (f) { return f(_p); }); });
            }
        ]).some(function (f) { return f(placeholder); })) {
            throw new TypeError(log_prefix_1.default + " " + JSON.stringify(placeholder) + " was passed as a value but only numbers and Animated.Value's are accepted.");
        }
    };
    for (var _i = 0, placeholders_1 = placeholders; _i < placeholders_1.length; _i++) {
        var placeholder = placeholders_1[_i];
        _loop_1(placeholder);
    }
};
exports.nativeFormula = function (args) {
    var placeholders = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        placeholders[_i - 1] = arguments[_i];
    }
    var argArray = args || [];
    validateArgs(placeholders);
    var _a = makeAst.apply(void 0, [argArray].concat(placeholders)), ast = _a[0], variables = _a[1];
    var result = reduce_ast_1.default(ast, variables, 'native');
    if (Array.isArray(result)) {
        throw new InvalidExpressionError_1.InvalidExpressionError('Result is an array');
    }
    return result;
};
var formula = function (args) {
    var placeholders = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        placeholders[_i - 1] = arguments[_i];
    }
    var argArray = args || [];
    validateArgs(placeholders);
    try {
        var _a = makeAst.apply(void 0, [argArray].concat(placeholders)), ast = _a[0], variables = _a[1];
        var result = reduce_ast_1.default(ast, variables, 'reanimated');
        if (Array.isArray(result)) {
            throw new InvalidExpressionError_1.InvalidExpressionError('Result is an array');
        }
        return result;
    }
    catch (err) {
        if (err.name === 'InvalidExpressionError') {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Expression " + (argArray.length > 0
                ? argArray.join('<variable>')
                : placeholders.join(',')) + " could not be parsed: " + err.message);
        }
        else {
            throw err;
        }
    }
};
exports.default = formula;
