"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_native_reanimated_1 = __importDefault(require("react-native-reanimated"));
var InvalidExpressionError_1 = require("./InvalidExpressionError");
var types_1 = __importDefault(require("./simple-math-ast/types"));
var log_prefix_1 = __importDefault(require("./log-prefix"));
var reduceAst = function (tree, variables, mathType) {
    if (!tree) {
        throw new InvalidExpressionError_1.InvalidExpressionError('Could not parse');
    }
    if (tree.token.type === types_1.default.NUMBER) {
        return Number(tree.token.value);
    }
    if (tree.token.type === types_1.default.CONSTANT) {
        if (tree.token.value === 'pi' || tree.token.value === 'PI') {
            return Math.PI;
        }
    }
    if (tree.token.type === types_1.default.VARIABLE) {
        return variables[tree.token.value];
    }
    var left = reduceAst(tree.left, variables, mathType);
    if (tree.token.type === types_1.default.NAMED_FUNCTION) {
        if (tree.token.value === 'log') {
            if (mathType === 'reanimated') {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " log() is not supported by Reanimated");
            }
            return Math.log(left);
        }
        if (tree.token.value === 'sin') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to sin()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.sin(left);
            }
            return Math.sin(left);
        }
        if (tree.token.value === 'exp') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to exp()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.exp(left);
            }
            return Math.exp(left);
        }
        if (tree.token.value === 'cos') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to cos()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.cos(left);
            }
            return Math.cos(left);
        }
        if (tree.token.value === 'tan') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to tan()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.tan(left);
            }
            return Math.tan(left);
        }
        if (tree.token.value === 'asin') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to asin()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.asin(left);
            }
            return Math.asin(left);
        }
        if (tree.token.value === 'atan') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to atan()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.atan(left);
            }
            return Math.atan(left);
        }
        if (tree.token.value === 'acos') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to acos()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.acos(left);
            }
            return Math.acos(left);
        }
        if (tree.token.value === 'round') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to round()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.round(left);
            }
            return Math.round(left);
        }
        if (tree.token.value === 'ceil') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to ceil()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.ceil(left);
            }
            return Math.ceil(left);
        }
        if (tree.token.value === 'floor') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to floor()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.floor(left);
            }
            return Math.floor(left);
        }
        if (tree.token.value === 'abs') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to abs()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.abs(left);
            }
            return Math.abs(left);
        }
        if (tree.token.value === 'deg') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to deg()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.concat(left, 'deg');
            }
            return left;
        }
        if (tree.token.value === 'diff') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to diff()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.diff(left);
            }
            throw new TypeError('diff() is not supported by native JS math');
        }
        if (tree.token.value === 'acc') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to acc()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.acc(left);
            }
            throw new TypeError('acc() is not supported by native JS math');
        }
        if (tree.token.value === 'defined') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to defined()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.defined(left);
            }
            throw new TypeError('defined() is not supported by native JS math');
        }
        if (tree.token.value === 'cot') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to cot()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.divide(1, react_native_reanimated_1.default.tan(left));
            }
            return 1 / Math.tan(left);
        }
        if (tree.token.value === 'sqrt') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot pass an array to sqrt()");
            }
            if (mathType === 'reanimated') {
                return react_native_reanimated_1.default.sqrt(left);
            }
            return Math.sqrt(left);
        }
        if (tree.token.value === 'min') {
            if (!Array.isArray(left) || left.length < 2) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Must pass multiple values to min()");
            }
            if (mathType === 'reanimated') {
                // @ts-ignore
                return react_native_reanimated_1.default.min.apply(react_native_reanimated_1.default, left);
            }
            return Math.min.apply(Math, left);
        }
        if (tree.token.value === 'max') {
            if (!Array.isArray(left) || left.length < 2) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Must pass multiple values to max()");
            }
            if (mathType === 'reanimated') {
                // @ts-ignore
                return react_native_reanimated_1.default.max.apply(react_native_reanimated_1.default, left);
            }
            return Math.max.apply(Math, left);
        }
        if (tree.token.value === 'set') {
            if (!Array.isArray(left) || left.length !== 2) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Must pass multiple two values to set()");
            }
            if (mathType === 'reanimated') {
                // @ts-ignore
                return react_native_reanimated_1.default.set(left[0], left[1]);
            }
            return left[1];
        }
        if (tree.token.value === 'pow') {
            if (!Array.isArray(left) || left.length < 2) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Must pass multiple values to pow()");
            }
            if (mathType === 'reanimated') {
                // @ts-ignore
                return react_native_reanimated_1.default.pow.apply(react_native_reanimated_1.default, left);
            }
            return left.reduce(function (b, c, index) { return (index > 0 ? Math.pow(b, c) : c); }, 0);
        }
        if (tree.token.value === '!') {
            if (Array.isArray(left)) {
                throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"!\" on array");
            }
            if (mathType === 'reanimated') {
                // @ts-ignore
                return react_native_reanimated_1.default.not(left);
            }
            return Number(!left);
        }
    }
    var right = reduceAst(tree.right, variables, mathType);
    if (tree.token.value === '+') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"+\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.add(left, right);
        }
        return left + right;
    }
    if (tree.token.value === '-') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"-\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.sub(left, right);
        }
        return left - right;
    }
    if (tree.token.value === '/') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"/\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.divide(left, right);
        }
        return left / right;
    }
    if (tree.token.value === '**') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"**\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.pow(left, right);
        }
        return Math.pow(left, right);
    }
    if (tree.token.value === '*') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"*\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.multiply(left, right);
        }
        return left * right;
    }
    if (tree.token.value === '%') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"%\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.modulo(left, right);
        }
        return left % right;
    }
    if (tree.token.value === '||') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"||\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.or(left, right);
        }
        return left || right;
    }
    if (tree.token.value === '&&') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"&&\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.and(left, right);
        }
        return left && right;
    }
    if (tree.token.value === '<') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"<\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.lessThan(left, right);
        }
        return Number(left < right);
    }
    if (tree.token.value === '>') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \">\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.greaterThan(left, right);
        }
        return Number(left > right);
    }
    if (tree.token.value === '>=') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \">=\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.greaterOrEq(left, right);
        }
        return Number(left >= right);
    }
    if (tree.token.value === '<=') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"<=\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.lessOrEq(left, right);
        }
        return Number(left <= right);
    }
    if (tree.token.value === '===') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"===\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.eq(left, right);
        }
        return Number(left === right);
    }
    if (tree.token.value === '!==') {
        if (Array.isArray(left) || Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError(log_prefix_1.default + " Cannot use operator \"!==\" on array");
        }
        if (mathType === 'reanimated') {
            return react_native_reanimated_1.default.neq(left, right);
        }
        return Number(left !== right);
    }
    if (tree.token.value === ',') {
        return (Array.isArray(left) ? left : [left]).concat((Array.isArray(right) ? right : [right]));
    }
    if (tree.token.value === ':') {
        if (Array.isArray(left)) {
            throw new InvalidExpressionError_1.InvalidExpressionError('Left expression of ternary expression cannot be an array');
        }
        if (Array.isArray(right)) {
            throw new InvalidExpressionError_1.InvalidExpressionError('Right expression of ternary expression cannot be an array');
        }
        return [left, right];
    }
    if (tree.token.value === '?') {
        if (!Array.isArray(right) || right.length !== 2) {
            throw new InvalidExpressionError_1.InvalidExpressionError('Right value of ? operator must be an array with 2 elements');
        }
        if (Array.isArray(left)) {
            throw new InvalidExpressionError_1.InvalidExpressionError('Left value of ? operator must cannot be an array');
        }
        return react_native_reanimated_1.default.cond(left, right[0], right[1]);
    }
    throw new Error("Could not parse! Operator: " + tree.token.value);
};
exports.default = reduceAst;
