"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = require("ramda");
var queries_1 = require("../queries");
var node_1 = __importDefault(require("./node"));
var InvalidExpressionError_1 = require("../../InvalidExpressionError");
var addOperandNode = function (nodes, token) {
    var node = new node_1.default(token);
    nodes.push(node);
};
var addOperatorNode = function (nodes, token) {
    var node = new node_1.default(token);
    if (token.args && token.args > 1) {
        node.setRight(nodes.pop() || null);
    }
    node.setLeft(nodes.pop() || null);
    nodes.push(node);
};
var parse = function (tokens, variables) {
    var ops = [];
    var nodes = [];
    ramda_1.forEach(function (token) {
        if (queries_1.isVariable(token) && !variables[token.value]) {
            throw new TypeError("Could not find variable " + token.value);
        }
        if (queries_1.isNumber(token) || queries_1.isVariable(token) || queries_1.isConstant(token)) {
            addOperandNode(nodes, token);
        }
        if (queries_1.isLeftParenthesis(token)) {
            ops.push(token);
        }
        if (queries_1.isRightParenthesis(token)) {
            while (ramda_1.last(ops) && !queries_1.isLeftParenthesis(ramda_1.last(ops))) {
                addOperatorNode(nodes, ops.pop());
            }
            ops.pop();
        }
        if (queries_1.isOperator(token) || queries_1.isNamedFunction(token)) {
            while (ramda_1.last(ops) &&
                ramda_1.last(ops).precedence >=
                    token.precedence &&
                !queries_1.isLeftParenthesis(token)) {
                addOperatorNode(nodes, ops.pop());
            }
            ops.push(token);
        }
    }, tokens);
    while (ops.length > 0) {
        addOperatorNode(nodes, ops.pop());
    }
    if (nodes.length > 1) {
        throw new InvalidExpressionError_1.InvalidExpressionError('Could not parse, too many nodes');
    }
    return nodes.pop();
};
exports.default = parse;
